/*
See LICENSE folder for this sample’s licensing information.

Abstract:
A struct representing a single day of weather data.
*/

public struct WeatherForecast: Codable {

    /// - Tag: PublicInit
    public init(daysFromNow: Int, forecast: Weather) {
        self.daysFromNow = daysFromNow
        self.forecast = forecast
    }

    /// The number of days in the future that this struct represents.
    public var daysFromNow: Int

    /// The number of days from now as a text description.
    public var daysFromNowDescription: String {
        switch daysFromNow {
        case 0: return "Today"
        case 1: return "Tomorrow"
        default: return "In \(daysFromNow) Days"
        }
    }

    /// The weather forecast for the day.
    public var forecast: Weather

}

/// Extends WeatherForecast with the ability to load and save data from a shared App Group container.

extension WeatherForecast {

    /// Loads weather forecast data from the App Group container.

    public static func loadSharedData() -> [ WeatherForecast ] {
        // In case this is the first time the app has been run, create some initial data.
        if FileManager.default.fileExists(atPath: sharedDataFileURL.path) == false {
            saveSharedData(data: randomForecastData())
        }

        let decoder = PropertyListDecoder()
        do {
            let data = try Data(contentsOf: sharedDataFileURL)
            return try decoder.decode([ WeatherForecast ].self, from: data)
        } catch {
            fatalError(error.localizedDescription)
        }
    }

    /// Saves weather forecast data from the App Group container.

    public static func saveSharedData(data: [ WeatherForecast ]) {
        let encoder = PropertyListEncoder()
        encoder.outputFormat = .xml

        do {
            let data = try encoder.encode(data)
            try data.write(to: sharedDataFileURL)
        } catch {
            fatalError(error.localizedDescription)
        }
    }

    /// Returns the URL of a data file in the shared App Group container.
    /// - Tag: SharedAppGroupDirectory
    static var sharedDataFileURL: URL {
        let appGroupIdentifier = "group.com.example.apple-samplecode.WeatherWidget"
        guard let url = FileManager.default.containerURL(forSecurityApplicationGroupIdentifier: appGroupIdentifier)
            else { preconditionFailure("Expected a valid app group container") }
        return url.appendingPathComponent("Data.plist")
    }

}

extension WeatherForecast {

    public static func randomForecastData() -> [ WeatherForecast ] {

        var data: [ WeatherForecast ] = []
        for daysFromNow in 0...4 {
            guard let randomWeather = Weather(rawValue: Int(arc4random_uniform(UInt32(Weather.allValues.count))))
                else { preconditionFailure("Expected a valid Weather enum") }
            data.append(WeatherForecast(daysFromNow: daysFromNow, forecast: randomWeather))
        }
        return data
    }

}
