/*
See LICENSE folder for this sample’s licensing information.

Abstract:
The widget view controller to show either today's weather forecast or a whole week of forecasts.
*/

import UIKit
import NotificationCenter
import WeatherFramework

class ForecastExtensionViewController: UITableViewController, NCWidgetProviding {

    var weatherForecastData = WeatherForecast.loadSharedData()

    // MARK: - View lifecycle

    override func viewDidLoad() {
        super.viewDidLoad()

        // Allow the today widget to be expanded or contracted.
        extensionContext?.widgetLargestAvailableDisplayMode = .expanded

        // Register the table view cell.
        let weatherForecastTableViewCellNib = UINib(nibName: "ForecastTableViewCell", bundle: nil)
        tableView.register(weatherForecastTableViewCellNib, forCellReuseIdentifier: ForecastTableViewCell.reuseIdentifier)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        tableView.reloadData()
    }
    
    // MARK: - Widget provider protocol

    /// - Tag: WidgetPerformUpdate
    func widgetPerformUpdate(completionHandler: (@escaping (NCUpdateResult) -> Void)) {
        // Reload the data from disk.
        weatherForecastData = WeatherForecast.loadSharedData()

        completionHandler(NCUpdateResult.newData)
    }

    /// - Tag: ActiveDisplayModeDidChange
    func widgetActiveDisplayModeDidChange(_ activeDisplayMode: NCWidgetDisplayMode, withMaximumSize maxSize: CGSize) {
        switch activeDisplayMode {
        case .compact:
            // The compact view is a fixed size.
            preferredContentSize = maxSize
        case .expanded:
            // Dynamically calculate the height of the cells for the extended height.
            var height: CGFloat = 0
            for index in weatherForecastData.indices {
                switch index {
                case 0: height += ForecastTableViewCell.todayCellHeight
                default: height += ForecastTableViewCell.standardCellHeight
                }
            }
            preferredContentSize = CGSize(width: maxSize.width, height: min(height, maxSize.height))
        @unknown default:
            preconditionFailure("Unexpected value for activeDisplayMode.")
        }
    }

    // MARK: - Content container protocol

    /// - Tag: ViewWillTransitionToSizeWithCoordinator
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)

        let updatedVisibleCellCount = numberOfTableRowsToDisplay()
        let currentVisibleCellCount = self.tableView.visibleCells.count
        let cellCountDifference = updatedVisibleCellCount - currentVisibleCellCount

        // If the number of visible cells has changed, animate them in/out along with the resize animation.
        if cellCountDifference != 0 {
            coordinator.animate(alongsideTransition: { [unowned self] (UIViewControllerTransitionCoordinatorContext) in
                self.tableView.performBatchUpdates({ [unowned self] in
                    // Build an array of IndexPath objects representing the rows to be inserted or deleted.
                    let range = (1...abs(cellCountDifference))
                    let indexPaths = range.map({ (index) -> IndexPath in
                        return IndexPath(row: index, section: 0)
                    })

                    // Animate the insertion or deletion of the rows.
                    if cellCountDifference > 0 {
                        self.tableView.insertRows(at: indexPaths, with: .fade)
                    } else {
                        self.tableView.deleteRows(at: indexPaths, with: .fade)
                    }
                }, completion: nil)
            }, completion: nil)
        }
    }

    // MARK: - Table view data source

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return numberOfTableRowsToDisplay()
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: ForecastTableViewCell.reuseIdentifier,
                                                       for: indexPath) as? ForecastTableViewCell
        else { preconditionFailure("Expected to dequeue a ForecastWidgetCell") }

        let weatherForecast = weatherForecastData[indexPath.row]
        cell.dateLabel.text = weatherForecast.daysFromNowDescription
        cell.forecastImageView.image = weatherForecast.forecast.imageAsset
        cell.forecastLabel.text = weatherForecast.forecast.description
        return cell
    }

    // MARK: - Table view delegate

    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        switch indexPath.row {
        case 0: return ForecastTableViewCell.todayCellHeight
        default: return ForecastTableViewCell.standardCellHeight
        }
    }

    /// - Tag: OpenMainApp
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        // Open the main app at the correct page for the day tapped in the widget.
        let weatherForecast = weatherForecastData[indexPath.row]
        if let appURL = URL(string: "weatherwidget://?daysFromNow=\(weatherForecast.daysFromNow)") {
            extensionContext?.open(appURL, completionHandler: nil)
        }

        // Don't leave the today extension with a selected row.
        tableView.deselectRow(at: indexPath, animated: true)
    }

    // MARK: - Helpers
    func numberOfTableRowsToDisplay() -> Int {
        if extensionContext?.widgetActiveDisplayMode == NCWidgetDisplayMode.compact {
            return 1
        } else {
            return weatherForecastData.count
        }
    }

}
